#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "pldstr.h"
#include "replace-text.h"


#define RPLTXT_INITIALIZED 12345

struct RPLTXT_variable_node *RPLTXT_get_node( struct RPLTXT_variable_node *n, char *varname )
{
	while (n != NULL)
	{
		if (strcmp(n->name, varname)==0) break;
		n = n->next;
	}

	return n;
}

/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_append_node
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_variable_node *n, 
	2.  char *value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_append_node( struct RPLTXT_variable_node *n, char *value )
{
	int newsize;
	char *buffer;

	newsize = strlen(value) +n->value_len;
	buffer = malloc( sizeof(char) *(newsize +1));
	if (buffer == NULL) return -1;

	snprintf(buffer, newsize +1,"%s%s",n->value,value);
	free(n->value);
	n->value = buffer;
	n->value_len = newsize;

	return 0;

}


/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_add_variable_node
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_variable_node **head, 
	2.  char *varname, 
	3.  char *value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_add_variable_node( struct RPLTXT_variable_node **head, char *varname, char *value )
{
	struct RPLTXT_variable_node *p;
	struct RPLTXT_variable_node *n = *head;
	struct RPLTXT_variable_node *new_node = malloc(sizeof(struct RPLTXT_variable_node));

	p = RPLTXT_get_node( (*head), varname );

	if (p != NULL)
	{
		RPLTXT_append_node( p, value );
		return 0;
	} 


	new_node->name = strdup(varname);
	new_node->name_len = strlen(new_node->name);
	new_node->value = strdup(value);
	new_node->value_len = strlen(new_node->value);

	new_node->next = NULL;

	if (n == NULL)
	{
		*head = new_node;
	} else {

		while (n->next != NULL)
		{
			n = n->next;
		}

		if (n->next == NULL)
		{
			n->next = new_node;
		}
	}

	return 0;

}


/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_addi_variable_node
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_variable_node **head, 
	2.  char *name, 
	3.  int value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_addi_variable_node( struct RPLTXT_variable_node **head, char *name, int value )
{
	char buffer[128];

	snprintf(buffer, sizeof(buffer), "%d", value);
	RPLTXT_add_variable_node( head, name, buffer );

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_addf_variable_node
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_variable_node **head, 
	2.  char *name, 
	3.  double value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_addf_variable_node( struct RPLTXT_variable_node **head, char *name, double value )
{
	char buffer[128];

	snprintf(buffer, sizeof(buffer), "%f", value);
	RPLTXT_add_variable_node( head, name, buffer );

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_add_variable
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_object *rpl, 
	2.  char *name, 
	3.  char *value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_add_variable( struct RPLTXT_object *rpl, char *name, char *value )
{
	RPLTXT_add_variable_node( &(rpl->variables), name, value );

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: *RPLTXT_getvalue
 Returns Type	: char
 	----Parameter List
	1. struct RPLTXT_variable_node **head, 
	2.  char *name , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
char *RPLTXT_getvalue( struct RPLTXT_variable_node **head, char *name )
{
	struct RPLTXT_variable_node *p;

	p = *head;

	while (p != NULL)
	{
		if (strncmp(p->name, name, p->name_len) == 0)
		{
			return p->value;
		}
		p = p->next;
	}

	return NULL;
}


/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_init
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_object *rpl , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_init( struct RPLTXT_object *rpl )
{
	rpl->variables = NULL;
	rpl->is_initialized = RPLTXT_INITIALIZED;

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: RPLTXT_done
 Returns Type	: int
 	----Parameter List
	1. struct RPLTXT_object *rpl , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int RPLTXT_done( struct RPLTXT_object *rpl )
{
	struct RPLTXT_variable_node *n, *nn;

	n = rpl->variables;
	while (n != NULL)
	{
		nn = n->next;
		if (n->value) free(n->value);
		if (n->name) free(n->name);
		if (n) free(n);
		n = nn;
	}

	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: *RPLTXT_convert
 Returns Type	: char
 	----Parameter List
	1. struct RPLTXT_object *rpl, 
	2.  char *source , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
char *RPLTXT_convert( struct RPLTXT_object *rpl, char *source )
{
	struct RPLTXT_variable_node *n;

	if (rpl->is_initialized != RPLTXT_INITIALIZED) return NULL;

	rpl->output_text = strdup(source);

	n = rpl->variables;

	while (n != NULL)
	{
//		printf("Search replacing '%s' with '%s'\n", n->name, n->value);
		PLD_strreplace( &(rpl->output_text), n->name, n->value, 0 );
		n = n->next;
	}

	return rpl->output_text;

}


